## -*-cperl-*-
## Filename: UCS/R/RSperl.pm
## Modified: Wed Aug 17 23:50:26 2005 (severt)   
##   Author: Stefan Evert
##  Purpose: communicate with R backend through RSPerl library

package UCS::R::RSPerl;

use UCS;
use UCS::File;

use lib '/Library/Frameworks/R.framework/Resources/library/RSPerl/share/lib/perl5/site_perl';	 # in-place edit by Install.perl
use R;
use RReferences;

$ENV{'R_HOME'} = '/Library/Frameworks/R.framework/Resources';	 # in-place edit by Install.perl

use base Exporter;
@EXPORT = qw(Start Stop Exec LoadVector DumpVector);

$R = undef;					# Expect object for R server process

# start R environment
sub Start ( ) {
  return 1 if defined $R;			# already running

  UCS::Status "Starting R environment ... ";
  R::initR("--silent", "--no-save");
  R::setDebug(0);		# just to be sure ...
  R::library("RSPerl");

  R::call("sink", "/dev/null");	# suppress all direct output to stdout (with print() and cat())
  R::eval("options(digits=22)"); # maximal accuracy for floating-point values (e.g. in DumpVector())

  R::call("source", "$UCS::R_BaseDir/R/lib/ucs.R");
  $ok = R::call("ucs.library", "sfunc");

  UCS::Die "Couldn't load R libraries: $error\n"
      unless $ok;
  
  UCS::Status "R has been set up successfully.\n";

  $R = 1;
  return 1;
}

# leave R environment (not possible with RSPerl)
sub Stop ( ) {
  return 1 unless defined $R;			# no server running
  
  UCS::Warn "Can't stop R server in RSPerl implementation!";

  return 1;
}

# @results = UCS::R::Exec("$cmd");
# execute R command and return result (which is usually a vector)
sub Exec ( $ ) {
  my $cmd = shift;
  Start() unless defined $R;			# auto-start R server when necessary
  $cmd =~ tr[\n][ ]s;				# command should be on single line;
  return R::eval($cmd);
}

# UCS::R::LoadVector($varname, \@data);
# load numeric vector into R through temporary file, using the scan() function
sub LoadVector ( $$ ) {
  my $var = shift;
  my $data = shift;
  my $l = scalar @$data;
  if ($l == 0) {
    Exec("$var <<- numeric(0)"); # ensure that we're writing to global namespace
  }
  else {
    my $tf = new UCS::File::Temp;
    for my $i (0 .. $l - 1) {
      $tf->write($data->[$i], "\n");
    }
    $tf->finish;
    my $filename = $tf->name;
    Exec("$var <<- scan(file=\"$filename\", quiet=TRUE, sep=\"\\n\", n=$l)");
    $tf->close;
  }
  my $l2 = Exec("length($var)");
  UCS::Die "UCS::R::LoadVector(): load failed (sent $l entries, received $l2)"
      unless $l2 == $l;
}

# $arrayref = UCS::R::DumpVector($varname);
# read numeric vector from R through temporary file, using the write() function;
# the data are returned as an anonymous array reference for efficiency reasons
sub DumpVector ( $ ) {
  my $var = shift;
  my @data = ();
  my $l = Exec("length($var)");
  if ($l == 0) {
    return [];
  }
  else {
    my $tf = new UCS::File::Temp;
    $tf->finish;
    my $filename = $tf->name;
    Exec("write($var, file=\"$filename\", ncolumns=1)");
    $data[$l-1] = 0;				# pre-allocate array
    my $size = 0;
    while (my $line = $tf->read) {
      chomp $line;
      $data[$size] = $line;
      $size++;
    }
    $tf->close;
    UCS::Die "UCS::R::DumpVector(): dump failed (expected $l entries, got $size)"
	unless $size == $l;
    return \@data;
  }
}

1;

__END__

=head1 NAME

UCS::R::RSPerl - RSPerl-based implementation of R backend

=head1 SYNOPSIS

  use UCS::R::RSPerl;
  ## exports Start(), Stop() and Exec() functions into current namespace
  ## as well as LoadVector() and DumpVector()


=head1 DESCRIPTION

This module should I<only> be used implicitly through B<UCS::R>, which loads
the B<UCS::R::RSPerl> implementation if available, and falls back on the
inefficient B<UCS::R::Expect> implementation otherwise.

Note that C<use UCS::R::RSPerl> will fail if RSPerl support is not available,
causing the compilation of the Perl script to abort.

=head1 COPYRIGHT

Copyright (C) 2004-2005 Stefan Evert.

This software is provided AS IS and the author makes no warranty as to
its use and performance. You may use the software, redistribute and
modify it under the same terms as Perl itself.

=cut

